import 'dart:async';

import 'package:bloc/bloc.dart';
import 'package:connectivity_plus/connectivity_plus.dart';

import 'loadable_state_state.dart';

class LoadableStateBloc extends Bloc<void, LoadableStateState> {
  late StreamSubscription<List<ConnectivityResult>> _updateStream;

  LoadableStateBloc() : super(const LoadableStateState(connections: null)) {
    emitState(List<ConnectivityResult> v) => emit(state.copyWith(connections: v));

    Connectivity().checkConnectivity().then(emitState);
    _updateStream = Connectivity().onConnectivityChanged.listen(emitState);
  }

  bool connectivityStatusKnown() => state.connections != null;
  bool isConnected({bool? def}) => !(state.connections?.contains(ConnectivityResult.none) ?? def!);

  @override
  Future<void> close() {
    _updateStream.cancel();
    return super.close();
  }
}
