import 'package:bloc/bloc.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

import '../../utilityWidgets/loadableHydratedBloc/loadable_hydrated_bloc_event.dart';
import '../loadable_state.dart';
import 'loadable_state_background_loading.dart';
import 'loadable_state_error_bar.dart';
import 'loadable_state_primary_loading.dart';

class LoadableStateConsumer<TController extends Bloc<LoadableHydratedBlocEvent<TState>, LoadableState<TState>>, TState> extends StatelessWidget {
  final Widget Function(TState state, bool loading) child;
  const LoadableStateConsumer({required this.child, super.key});

  static Duration animationDuration = const Duration(milliseconds: 200);

  @override
  Widget build(BuildContext context) {
    var loadableState = context.watch<TController>().state;

    return Column(
      children: [
        LoadableStateErrorBar(visible: loadableState.showError()),
        Expanded(
          child: Stack(
            children: [
              LoadableStatePrimaryLoading(visible: loadableState.showPrimaryLoading()),
              LoadableStateBackgroundLoading(visible: loadableState.showBackgroundLoading()),

              AnimatedOpacity(
                  opacity: loadableState.showContent() ? 1.0 : 0.0,
                  duration: animationDuration,
                  curve: Curves.easeInOut,
                  child: loadableState.showContent() ? child(loadableState.data, loadableState.isLoading) : const SizedBox.shrink()
              ),
            ],
          ),
        )
      ],
    );
  }
}
