import 'package:hydrated_bloc/hydrated_bloc.dart';

import '../../repository/repository.dart';
import 'loadable_hydrated_bloc_event.dart';
import '../../loadableState/loadable_state.dart';

abstract class LoadableHydratedBloc<TEvent extends LoadableHydratedBlocEvent<TState>, TState, TRepository extends Repository<TState>> extends HydratedBloc<LoadableHydratedBlocEvent<TState>, LoadableState<TState>> {
  late TRepository _repository;
  LoadableHydratedBloc() : super(const LoadableState()) {
    on<Emit<TState>>((event, emit) => emit(LoadableState(isLoading: event.loading, data: event.state(innerState ?? fromNothing()))));
    on<ClearState<TState>>((event, emit) => emit(const LoadableState()));
    
    _repository = repository();
    loadState();
  }

  TState? get innerState => state.data;
  TRepository get repo => _repository;

  @override
  fromJson(Map<String, dynamic> json) => LoadableState(isLoading: true, data: fromStorage(json));
  @override
  Map<String, dynamic>? toJson(LoadableState<TState> state) => state.data == null ? {} : state.data.toJson();

  Future<void> loadState();
  TRepository repository();

  TState fromNothing();
  TState fromStorage(Map<String, dynamic> json);
  Map<String, dynamic>? toStorage(TState state);
}
