import 'dart:developer';

import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

import '../../../../../api/mhsl/message/getMessages/getMessagesResponse.dart';
import '../../../../../view/pages/more/message/messageView.dart';
import '../../../infrastructure/loadableState/loadable_state.dart';
import '../../../infrastructure/loadableState/view/loadable_state_consumer.dart';
import '../../../infrastructure/utilityWidgets/bloc_module.dart';
import '../../../infrastructure/utilityWidgets/loadableHydratedBloc/loadable_hydrated_bloc_event.dart';
import '../bloc/marianum_message_bloc.dart';
import '../bloc/marianum_message_state.dart';

class MarianumMessageListView extends StatelessWidget {
  const MarianumMessageListView({super.key});

  @override
  Widget build(BuildContext context) => BlocModule<MarianumMessageBloc, LoadableState<MarianumMessageState>>(
    create: (context) => MarianumMessageBloc(),
    child: (context, state) {
      // if(value.primaryLoading()) return const LoadingSpinner();
      log(state.toString());
      return Scaffold(
        appBar: AppBar(
          title: const Text('Marianum Message'),
          actions: [
            IconButton(onPressed: () => context.read<MarianumMessageBloc>().add(MessageEvent()), icon: Icon(Icons.abc)),
            IconButton(onPressed: () => context.read<MarianumMessageBloc>().add(Emit((state) => MarianumMessageState(messageList: MarianumMessageList(base: "", messages: [MarianumMessage(url: "", name: "Teeest", date: "now")])))), icon: Icon(Icons.add)),
            IconButton(onPressed: () => context.read<MarianumMessageBloc>().add(ClearState()), icon: Icon(Icons.add))
          ],
        ),
        body: LoadableStateConsumer<MarianumMessageBloc, MarianumMessageState>(
          child: (state, loading) => ListView.builder(
            itemCount: state.messageList.messages.length,
            itemBuilder: (context, index) {
              var message = state.messageList.messages.toList()[index];
              return ListTile(
                leading: const Column(
                  mainAxisAlignment: MainAxisAlignment.center,
                  children: [Icon(Icons.newspaper)],
                ),
                title: Text("${message.name}${loading ? "loading" : "notloading"}", overflow: TextOverflow.ellipsis),
                subtitle: Text('vom ${message.date}'),
                trailing: const Icon(Icons.arrow_right),
                onTap: () {
                  Navigator.push(context, MaterialPageRoute(builder: (context) => MessageView(basePath: state.messageList.base, message: message as GetMessagesResponseObject)));
                },
              );
            }
          ),
        ),
      );
    }
  );
}
